"""
duration.py

measure the duration using the coda envelope
"""

import obspy
import types
import warnings
import numpy as np
import pandas as pd
import madpy.noise as n
from typing import Tuple
import madpy.checks as ch
import madpy.config as config
import matplotlib.pyplot as plt
from scipy.signal import hilbert
import madpy.plotting.dur as plot
from threadpoolctl import threadpool_limits
from scipy.optimize import lsq_linear as lsq


def measure_duration(
    st: obspy.Stream, 
    cfg: types.ModuleType = config, 
    ptype: str ='log', 
) -> pd.DataFrame:
    """Measure noise level and duration

    Args:
        st: stream containing one or more time series
        cfg: configuration file
        ptype: plot type [linear, log]
        
    Returns:
        df: dataframe of time series duration information

    """    
    
    output = []   
    for tr in st:
        
        preliminary_checks(tr, ptype, cfg)
        noise = n.rms_noise(tr.copy(), 'duration', cfg)
        duration, cc = coda_duration(tr.copy(), noise, ptype, cfg)
        output.append([str(tr.stats.o.date), str(tr.stats.o.time)[:11],
                       tr.stats.network, tr.stats.station, tr.stats.channel,
                       duration, cc, np.log10(noise)])

    df = format_output(output)
    if cfg.Duration.save_output:
        df.to_csv(f'{cfg.Duration.output_path}/dur-output.csv', 
                  float_format='%0.5f', index=False)
        
    return df


def preliminary_checks(
    tr: obspy.Trace, 
    ptype: str, 
    cfg: types.ModuleType = config
) -> None:
    """Make sure all necessary information is present
    
    This function checks...
    1. The configuration file is setup correctly
    2. The trace has all relevant information
    3. There is sufficient time series data
    
    Args:
        tr: time series
        ptype: plot type
        cfg: configuration file
        
    Returns:
        None

    """
    
    ch.check_config(cfg.Duration())
    ch.check_waveform(tr)
    ch.check_plottype(ptype)       
        
    return None
            

def coda_duration(
    tr: obspy.Trace, 
    noise: float, 
    ptype: str, 
    cfg: types.ModuleType = config
) -> Tuple[float, float]:    
    """Measure duration and associate quality control
    
    Args:
        tr: time series
        noise: noise level
        ptype: plot type 
        cfg: configuration file
        
    Returns:
        dur: duration in seconds
        cc: correlation coefficient
    
    """

    dcfg = cfg.Duration()
    avg_time, avg_data_lin = moving_average(tr, dcfg)
    avg_data_log = log_envelope(avg_data_lin)
    fit_begin, fit_end = coda_fitting_window(tr, dcfg, avg_time, 
                                             avg_data_log, noise)
    coda_line = coda_line_fit(avg_time[fit_begin:fit_end], 
                              avg_data_log[fit_begin:fit_end])
    dur, i_dur = get_duration_index(dcfg, coda_line, noise)
    cc = get_correlation_coefficient(coda_line, avg_time, avg_data_log, 
                                     fit_begin, i_dur)
    
    if dcfg.plot:
        fig = plot.duration_plot(tr, avg_time, avg_data_lin, avg_data_log, 
                                 fit_begin, fit_end, dur, cc, coda_line, 
                                 noise, ptype, dcfg)
   
    return dur, cc


def moving_average(
    tr: obspy.Trace, 
    dcfg: config.Duration = config.Duration
) -> Tuple[np.ndarray, np.ndarray]:
    """Perform a moving average of the data
    
    Args:
        tr: time series
        dcfg: duration instance of configuration file
        
    Returns:
        averaged_time: moving average of time axis
        averaged_data: moving average of time series 
    
    """
    
    if dcfg.moving_average_window == 0:
        averaged_data = tr.data
        averaged_time = time_relative_p(tr)
    else:
        mov_avg_wind_samp = dcfg.moving_average_window * \
                            int(tr.stats.sampling_rate)
        time_wrt_p = time_relative_p(tr)
        time_averaging = pd.Series(time_wrt_p).rolling(
                         window=mov_avg_wind_samp).median()
        averaged_time = time_averaging.iloc[mov_avg_wind_samp-1:].values
        data_averaging = pd.Series(np.abs(hilbert(tr.data))).rolling(
                         window=mov_avg_wind_samp).mean()
        averaged_data = data_averaging.iloc[mov_avg_wind_samp-1:].values
    
    return averaged_time, averaged_data


def time_relative_p(tr: obspy.Trace) -> np.ndarray:    
    """Get time axis with respect to P- arrival
    
    Args: 
        tr: time series
        
    Returns:
        shifted time axis with respect to P-

    """
    
    p_wrt_b = (tr.stats.o - tr.stats.starttime) + tr.stats.p
    
    return np.arange(0, len(tr.data)) * tr.stats.delta - p_wrt_b


def log_envelope(data: np.ndarray) -> np.ndarray:    
    """Take log of coda envelope
    
    Args:
        data: coda envelope
        
    Returns
        log_env: the log (base 10) of the coda envelope
        
    """
    
    data_pos = np.where(data > 0, data, np.nan)
    log_env = np.log10(data_pos)
    log_env[np.isinf(log_env)] = np.nan
    log_env[np.isneginf(log_env)] = np.nan
    
    return log_env


def coda_fitting_window(
    tr: obspy.Trace, 
    dcfg: config.Duration, 
    time: np.ndarray, 
    data: np.ndarray, 
    noise: float
) -> Tuple[int, int]:  
    """Find coda search window by index
    
    Args:
        tr: time series
        dcfg: duration instance of configuration file
        time: averaged time axis
        data: averaged coda envelope data
        noise: noise level
        
    Returns:
        i_begin: search window beginning index
        i_end: search window ending index
        
    """
    
    search_window_begin = search_window(tr, time, dcfg, 'begin')
    search_window_end = search_window(tr, time, dcfg, 'end')
    i_max = np.where(data == np.nanmax(
            data[search_window_begin:search_window_end]))[0][0]
    i_end = fitting_window_end(tr, dcfg, data, i_max, noise)
    i_begin = fitting_window_start(dcfg, i_max, i_end)
    
    return i_begin, i_end


def search_window(
    tr: obspy.Trace, 
    time: np.ndarray, 
    dcfg: config.Duration, 
    position: str
) -> int:    
    """Find indices for coda search window
    
    Args:
        tr: time series
        time: time axis
        dcfg: duration instance of configuration file
        position: specify the part of the search window [begin, end]
        
    Returns:
        search_index: index value of desired search window position
    
    """
    
    search_seconds = search_window_seconds(tr, dcfg, position)
    search_index = np.where(np.sign(time - search_seconds) == 1)[0][0]
    
    return search_index


def search_window_seconds(
    tr: obspy.Trace, 
    dcfg: config.Duration, 
    position: str
) -> float:   
    """Get search window in terms of seconds
    
    Args:
        tr: time series
        dcfg: duration instance of configuration file
        position: specify the part of the search window [begin, end]
        
    Returns:
        seconds_wrt_p: time value of desired search window position
        
    Raises:
        AssertionError: if position is not begin or end
        
    """
    
    phase_p = phase_relative_p(tr, dcfg.signal_phase)
    
    assert position in ['begin', 'end'], \
        f'(ValueError) Position {position} unrecognized'
    if position == 'begin':
        seconds_wrt_p = phase_p + dcfg.signal_window_begin
    elif position == 'end':
        seconds_wrt_p = phase_p + dcfg.signal_window_end
    
    return seconds_wrt_p

        
def phase_relative_p(tr: obspy.Trace, phase: str) -> obspy.UTCDateTime:    
    """Get phase arrival in seconds relative to the P- arrival
    
    Args:
        tr: time series
        phase: the seismic phase
        
    Returns:
        phase_time: seconds format for phase        
    
    """
    
    if phase == 'O':
        phase_time = -tr.stats.p
    elif phase == 'P':
        phase_time = tr.stats.p - tr.stats.p
    elif phase == 'S':
        phase_time = tr.stats.s - tr.stats.p
        
    return phase_time


def fitting_window_start(
    dcfg: config.Duration, 
    i_max: int, 
    i_end: int
) -> int:   
    """Determine the start of the line fitting window
    
    Args:
        dcfg: duration instance of configuration file
        i_max: index of search window beginning
        i_end: index of search window end
        
    Returns:
        i_begin: index of fitting window beginning
        
    """
    
    if dcfg.start_fit_max > 1:
        i_begin = (i_end - i_max) / dcfg.start_fit_max + i_max
    else:
        i_begin = np.copy(i_max)
        
    return int(i_begin)


def fitting_window_end(
    tr: obspy.Trace, 
    dcfg: config.Duration, 
    data: np.ndarray, 
    i_max: int, 
    noise: float
) -> int:                                   
    """Find index of desired end-of-coda threshold
    
    Args: 
        tr: time series
        dcfg: duration instance of configuration file
        data: averaged coda envelope data
        i_max: index of search window beginning
        noise: noise level
        
    Returns:
        index of fitting window end
        
    Raises:
        AssertionError: if the fitting window is less than the s-p time
        
    """
    
    noise_diff = data[i_max:] - np.log10(dcfg.end_fit_noise * noise)
    i_noise = np.where(np.sign(noise_diff) == -1)[0]
    ch.check_fitting_window_end(i_noise, i_max, 
                                tr.stats.delta, tr.stats.s - tr.stats.p)
    
    return int(i_noise[0] + i_max)


def coda_line_fit(x0: np.ndarray, d0: np.ndarray) -> np.ndarray:    
    """Does least square inversion 
    
    Note...
    1. The slope is forced be positive
    2. Numpy is not allowed to parallelize in the background
    
    Args:
        x0: time axis
        d0: averaged data
        
    Returns:
        m.x: the parameters of the inversion
        
    Raises:
        AssertionError: if either array is not a float
        AssertionError: if time axis has NaN values        
        
    """
    
    x, d = ch.check_coda(x0, d0)
    ones = np.ones(np.shape(x))
    G = np.stack((ones, x), axis=-1)
    
    with threadpool_limits(limits=1, user_api='blas'):
        m = lsq(G, d, bounds=([-np.inf, -np.inf], [np.inf, 0]))
        
    return m.x


def get_duration_index(
    dcfg: config.Duration, 
    line: np.ndarray, 
    noise: float
) -> Tuple[float, int]:    
    """Find duration and duration index
    
    Args:
        dcfg: duration instance of configuration file
        line: line fit coefficients
        noise: noise level
        
    Returns:
        dur: duration in seconds
        i_dur: duration index
        
    """
    
    x_extended, y_extended = extend_line(line[1], line[0], 0., 10000., 0.001) 
    threshold = coda_line_end(dcfg, noise)
    cross = np.where(np.sign(y_extended - threshold) == -1)[0]
    ch.check_duration_index(cross)
    dur = x_extended[cross[0]]
    i_dur = cross[0]
    
    return dur, i_dur


def extend_line(
    m: float, 
    b: float, 
    xmin: float, 
    xmax: float, 
    xint: float
) -> Tuple[np.ndarray, np.ndarray]:
    """Extends line given line parameters
    
    Args:
        m: slope
        b: intercept
        xmin: minimum of line extension
        xmax: minimum of line extension
        xint: interval of line extension
        
    Returns:
        x: extended x data
        y: extended y data
        
    """
    
    x = np.arange(xmin, xmax, xint)
    y = m * x + b
    
    return x, y                   
                    
    
def coda_line_end(dcfg: config.Duration, noise: float) -> float:   
    """Determine duration threshold
    
    Args:
        dcfg: duration instance of configuration file
        noise: noise level
        
    Returns:
        threshold: duration threshold
    
    """
    
    if dcfg.threshold_type == 'absolute':
        threshold = dcfg.duration_absolute_threshold
    elif dcfg.threshold_type == 'noise':
        threshold = dcfg.duration_noise_threshold * np.log10(noise)
        
    return threshold
 

def get_correlation_coefficient(
    line: np.ndarray, 
    time: np.ndarray, 
    data: np.ndarray, 
    data_begin: int, 
    line_end: int, 
    perc: float = 0.9
) -> np.ndarray:  
    """Format data to calculate correlation coefficient
    
    Args:
        line: line fit coefficients
        time: time axis
        data: averaged coda envelope data
        
    Returns:
        cc: correlation coefficient matrix
        
    Raises:
        UserWarning: if duration is longer than waveform segment
    
    """
    
    x_extended, y_extended = extend_line(line[1], line[0], 0, 10000, 0.001)
    line_begin = np.where(np.sign(x_extended - time[data_begin]) == 1)[0][0]
    data_end_0 = np.where(np.sign(time - x_extended[line_end]) == 1)[0]
    if len(data_end_0) == 0:
        data_end = int(np.around(perc * len(time)))
        warnings.warn(f'''Duration {x_extended[line_end]} 
                      longer than waveform  segment {time[-1]}. 
                      Fitting cc line to {perc * 100}% of waveform''')
    else:
        data_end = data_end_0[0]
    time_rounded = np.round(time, 3)
    _, i_line, i_data = np.intersect1d(x_extended[line_begin:line_end], 
                                       time_rounded[data_begin:data_end], 
                                       return_indices=True)
    x_data = time[i_data + data_begin]
    y_data = data[i_data + data_begin]
    x_line = x_extended[i_line + line_begin]
    y_line = y_extended[i_line + line_begin]
    xdata = np.column_stack((x_data, x_line))
    ydata = np.column_stack((10**y_data, 10**y_line))
    cc = calculate_cc(xdata, ydata)[0, 3]
    
    return cc
    
    
def calculate_cc(xdata: np.ndarray, ydata: np.ndarray) -> np.ndarray:   
    """Calculate Pearson correlation coefficient
    
    Args:
        xdata: waveform and line fit time axis
        ydata: waveform and line fit data axis
        
    Returns:
        cc: correlation matrix
        
    Raises:
        AssertionError: if correlation matrix is not real
        
    """
    
    cc = np.corrcoef(xdata, ydata, rowvar=False)
    ch.check_cc(cc, 0, 3)
    
    return cc   
       
    
def format_output(data: list) -> pd.DataFrame:
    """Turn list into dataframe
    
    Args:
        data: list of duration information
        
    Returns:
        df: dataframe of duration information
    
    Raises:
        AssertionError: if data size does not match column size
    
    """
    
    column_names = ['date', 'time', 'network', 'station', 'channel', 
                    'duration', 'cc', 'noise']
    assert len(data[0]) == len(column_names), \
        '(ValueError) Data length must match column length'
    df = pd.DataFrame(data, columns=column_names)
    
    return df


"""
amplitude.py

measure the maximum peak-to-peak amplitude
"""

import obspy
import types
import numpy as np
import pandas as pd
import madpy.noise as n
from typing import Tuple
import madpy.checks as ch
import madpy.config as config
import matplotlib.pyplot as plt
import madpy.plotting.amp as plot


def measure_amplitude(
    st: obspy.Stream, 
    cfg: types.ModuleType = config, 
) -> pd.DataFrame:
    """Measure noise level and amplitude

    Args:
        st: stream containing one or more time series
        cfg: configuration file
        
    Returns:
        df: dataframe of time series amplitude information

    """
    
    output = []
    for tr in st:
    
        preliminary_checks(tr, cfg)
        noise = n.rms_noise(tr, 'amplitude', cfg)
        amplitude = max_amplitude(tr, noise, cfg)
        output.append([str(tr.stats.o.date), str(tr.stats.o.time)[:11],
                       tr.stats.network, tr.stats.station, tr.stats.channel,
                       amplitude, noise])
        
    df = format_output(output)
    if cfg.Amplitude.save_output:
        df.to_csv(f'{cfg.Amplitude.output_path}/amp-output.csv', 
                  float_format='%0.5f', index=False)
        
    return df


def preliminary_checks(
    tr: obspy.Trace,  
    cfg: types.ModuleType = config
) -> None:
    """Make sure all necessary information is present
    
    This function checks...
    1. The configuration file is setup correctly
    2. The trace has all relevant information
    3. There is sufficient time series data
    
    Args:
        tr: time series
        cfg: configuration file
        
    Returns:
        None

    """
    
    ch.check_config(cfg.Amplitude())
    ch.check_waveform(tr)
        
    return None

        
def max_amplitude(
    tr: obspy.Trace, 
    noise: float, 
    cfg: types.ModuleType = config
) -> float:
    """Measure maximum peak-to-peak amplitude
    
    Args:
        tr: time series
        noise: noise level
        cfg: configuration file
    
    Returns: 
        amp: maximum peak-to-peak amplitude
        
    Raises:
        ValueError: if max amplitude is not real and positive
    
    """
    
    acfg = cfg.Amplitude()
    tr_signal = trim_waveform_signal(tr.copy())
    peaks_nan = inflection_points(tr_signal.data)
    peaks = remove_nan(peaks_nan)
    p2p_amplitudes = np.diff(peaks)
    amp = np.max(np.abs(p2p_amplitudes)) * acfg.amp_factor
    ch.check_amplitude(amp)
    
    if acfg.plot:
        indices = p2p_indices(tr_signal, peaks, p2p_amplitudes)
        plot.amplitude_plot(tr, tr_signal, amp, indices, noise, acfg)
        
    return amp


def trim_waveform_signal(
    tr: obspy.Trace, 
    cfg: types.ModuleType = config
) -> obspy.Trace:
    """Cut the time series to signal window
    
    Args:
        tr: time series
        cfg: configuration file
        
    Returns:
        tr: trimmed time series
    
    """
    
    starttime, endtime = signal_window(tr, cfg)
    tr.trim(starttime=starttime, endtime=endtime)
    
    return tr

    
def signal_window(
    tr: obspy.Trace, 
    cfg: types.ModuleType = config
) -> Tuple[obspy.UTCDateTime, obspy.UTCDateTime]:    
    """Get the starttimes and endtimes of signal window
    
    Args:
        tr: time series
        cfg: configuration file
        
    Returns:
        starttime: signal window beginning date
        endtime: signal window ending date
        
    Raises:
        AssertionError: Window begins before time series begins
        AssertionError: Window ends after time series ends
        
    """
    
    acfg = cfg.Amplitude()
    arrival = n.arrival_time_utc(tr, acfg.signal_phase)
    starttime = arrival + acfg.signal_window_begin
    endtime = arrival + acfg.signal_window_end
    ch.check_window(tr, starttime, endtime)
    
    return starttime, endtime


def inflection_points(data: np.ndarray) -> np.ndarray:    
    """Isolate the peaks of an array
    
    Args:
        data: time series
        
    Returns:
        inflection_points: peaks of the time series
    
    """
    
    nan_points = np.concatenate([[0], np.diff(np.sign(np.diff(data))), [0]])
    nan_points[nan_points == 0] = np.nan
    nan_points[~np.isnan(nan_points)] = 0
    inflection_points = nan_points + data
    
    return inflection_points


def remove_nan(array: np.ndarray) -> np.ndarray:   
    """Remove NaN values
    
    Args:
        array: time series
        
    Returns:
        the time series without NaN values
        
    """
    
    return array[~np.isnan(array)]


def p2p_indices(
    tr: obspy.Trace, 
    peaks: np.ndarray, 
    amplitudes: np.ndarray
) -> Tuple[float, float]:
    """Get peak indices of max peak-to-peak amplitude
    
    Args:
        tr: time series
        peaks: the inflection points of the time series
        amplitudes: values of each peak to peak
        
    Return:
        idx: indices of two peaks associated with maximum amplitude
        
    """
    
    i_diff = np.where(np.abs(amplitudes) == np.max(np.abs(amplitudes)))
    i_peak1 = i_diff[0][0]
    i_peak2 = i_peak1 + 1
    peak1 = peaks[i_peak1]
    peak2 = peaks[i_peak2]
    nan_points = inflection_points(tr.data)
    i_p1_0 = np.where(nan_points == peak1)
    i_p2_0 = np.where(nan_points == peak2)
    idx = p2p_indices_check(i_p1_0, i_p2_0)
    
    return idx


def p2p_indices_check(i_p1_0: float, i_p2_0: float) -> Tuple[float, float]:
    """Verify the indices are associated with the peaks
    
    Args:
        i_p1_0: preliminary peak 1
        i_p2_0: preliminary peak 2
        
    Returns:
        idx: final peaks
        
    """
    
    if len(i_p1_0[0]) > 1 or len(i_p2_0[0]) > 1:
        x_p1 = np.repeat(i_p1_0[0], len(i_p2_0[0]))
        x_p2 = np.tile(i_p2_0[0], len(i_p2_0[0]))
        x_p2_p1 = np.subtract(x_p2, x_p1)
        x_p2_p1 = np.divide(x_p2_p1, 1.0)
        x_p2_p1[x_p2_p1 < 0] = np.nan
        i_x = np.where(x_p2_p1 == np.nanmin(x_p2_p1))
        i_p1 = x_p1[int(i_x[0])]
        i_p2 = x_p2[int(i_x[0])]
        idx = np.array([i_p1, i_p2])
    else:
        i_p1 = i_p1_0[0]
        i_p2 = i_p2_0[0]
        idx = np.array([i_p1, i_p2])
        
    return idx


def format_output(data: list) -> pd.DataFrame:    
    """Turn list into dataframe
    
    Args:
        data: list of amplitude information
        
    Returns:
        df: dataframe of amplitude information
    
    Raises:
        AssertionError: if data size does not match column size
    
    """
    
    column_names = ['date', 'time', 'network', 'station', 'channel', 
                    'amplitude', 'noise']
    assert len(data[0]) == len(column_names), \
        '(ValueError) Data length must match column length'
    df = pd.DataFrame(data, columns=column_names)
    
    return df


import obspy
import unittest
import numpy as np
import madpy.noise as n
import madpy.tests.testdata.config as cfg


class TestNoise(unittest.TestCase):
    
    
    def test_rms_noise(self):
        
        st = obspy.read('testdata/*.mseed')
        for tr in st:
            tr.stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
            tr.stats.p = 10.
            tr.stats.s = 20.
 
        self.assertAlmostEqual(n.rms_noise(st[0], 'amplitude', cfg), 0.1, 1)
        self.assertAlmostEqual(n.rms_noise(st[0], 'duration', cfg), 0.1, 1)
        self.assertAlmostEqual(np.log10(n.rms_noise(st[1], 'amplitude')), -10, 0)
        self.assertAlmostEqual(np.log10(n.rms_noise(st[1], 'duration')), -10, 0)
        
        
    def test_trim_waveform_noise(self):
        
        st = obspy.read('testdata/*.mseed')
        for tr in st:
            tr.stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
            tr.stats.p = 10.
            tr.stats.s = 20.
            tra = n.trim_waveform_noise(tr.copy(), 'amplitude', cfg)
            trd = n.trim_waveform_noise(tr.copy(), 'duration', cfg)
            self.assertEqual(len(tra.data), 2901)
            self.assertEqual(len(trd.data), 1001)
            
            
    def test_noise_window(self):
        
        st = obspy.read('testdata/*.mseed')
        for tr in st:
            tr.stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
            tr.stats.p = 10.
            tr.stats.s = 20.
            starttime_amp, endtime_amp = n.noise_window(tr, 'amplitude', cfg)
            starttime_dur, endtime_dur = n.noise_window(tr, 'duration', cfg)
            noise_begin_amp = obspy.UTCDateTime('2020-10-10T13:04:40.00')
            noise_end_amp = obspy.UTCDateTime('2020-10-10T13:05:09.00')
            noise_begin_dur = obspy.UTCDateTime('2020-10-10T13:04:40.00')
            noise_end_dur = obspy.UTCDateTime('2020-10-10T13:04:50.00')
            self.assertEqual(starttime_amp, noise_begin_amp)
            self.assertEqual(endtime_amp, noise_end_amp)
            self.assertEqual(starttime_dur, noise_begin_dur)
            self.assertEqual(endtime_dur, noise_end_dur)
            
    def test_measurement_type(self):
        
        self.assertRaises(AssertionError, n.measurement_type, 'option')
        self.assertEqual(n.measurement_type('amplitude', cfg).noise_phase, 
                         cfg.Amplitude.noise_phase)
        self.assertEqual(n.measurement_type('duration', cfg).noise_phase, 
                         cfg.Duration.noise_phase)
                
        
    def test_arrival_time_utc(self):
        
        st = obspy.read('testdata/*.mseed')
        for tr in st:
            tr.stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
            tr.stats.p = 10.
            tr.stats.s = 20.
            p = obspy.UTCDateTime('2020-10-10T13:05:10.00')
            s = obspy.UTCDateTime('2020-10-10T13:05:20.00')
            self.assertEqual(n.arrival_time_utc(tr, 'O'), tr.stats.o)
            self.assertEqual(n.arrival_time_utc(tr, 'P'), p)
            self.assertEqual(n.arrival_time_utc(tr, 'S'), s)
            
    
    def test_root_mean_square(self):
        
        data = np.cos(np.arange(-np.pi, np.pi, np.pi / 12))
        self.assertAlmostEqual(n.root_mean_square(data), 1 / np.sqrt(2))
        data = np.ones((10,)) * 5
        self.assertEqual(n.root_mean_square(data), 5)
        
        
        
if __name__ == '__main__':
    unittest.main()


"""
checks.py

performs checks from noise, amplitude, and duration modules
"""

import obspy
import types
import numpy as np
import pandas as pd 


def check_config(c):   
    """Verify config file is in order"""
    
    if (hasattr(c, 'noise_phase') and
        hasattr(c, 'noise_window_begin') and 
        hasattr(c, 'noise_window_end') and
        hasattr(c, 'signal_phase') and 
        hasattr(c, 'signal_window_begin') and 
        hasattr(c, 'signal_window_end') and
        hasattr(c, 'plot')):
        
        check_common_config(c)
        
        if hasattr(c, 'amp_factor'):
            
            check_amplitude_config(c)
            
        elif (hasattr(c, 'moving_average_window') and
            hasattr(c, 'start_fit_max') and
            hasattr(c, 'end_fit_noise') and 
            hasattr(c, 'threshold_type') and
            hasattr(c, 'duration_noise_threshold') and 
            hasattr(c, 'duration_absolute_threshold')):
            
            check_duration_config(c)
            
    else:
        
        raise AttributeError('''Missing attributes common to 
                             class Amplitude and class Duration''')
        
    return None
                    
                    
def check_common_config(c):                   
    """Check attributes common to class Amplitude and class Duration"""
    
    assert isinstance(c.noise_phase, str), \
        f'''(TypeError) 
        Phase {c.noise_phase} must be a string'''
    assert c.noise_phase in ['O', 'P', 'S'], \
        f'''(ValueError) 
        Phase {c.noise_phase} not recognized'''
    assert isinstance(c.noise_window_begin/1., float), \
        f'''(TypeError) 
        Noise window start {c.noise_window_begin} must be a float'''
    assert isinstance(c.noise_window_end/1., float), \
        f'''(TypeError) 
        Noise window end {c.noise_window_end} must be a float'''
    assert c.noise_window_end - c.noise_window_begin > 0, \
        '''(ValueError) 
        Noise window length must be greater than zero'''
    assert isinstance(c.signal_phase, str), \
        f'''(TypeError) 
        Phase {c.signal_phase} must be a string'''
    assert c.signal_phase in ['O', 'P', 'S'], \
        f'''(ValueError) 
        Phase {c.signal_phase} not recognized'''
    assert isinstance(c.signal_window_begin/1., float), \
        f'''(TypeError) 
        Signal window start {c.signal_window_begin} must be a float'''
    assert isinstance(c.signal_window_end/1., float), \
        f'''(TypeError) 
        Signal window end {c.signal_window_end} must be a float'''
    assert c.signal_window_end - c.signal_window_begin > 0, \
        f'''(ValueError) 
        Noise window length must be greater than zero'''
    assert isinstance(c.save_output, bool), \
        f'''(TypeError)
        Save output {c.save_output} must be a boolean'''
    assert isinstance(c.output_path, str), \
        f'''(TypeError)
        Output path {c.output_path} must be a string'''
    if c.save_output:
        assert len(c.output_path) > 0, \
            f'''(ValueError) 
            Output path must be specified if save_output=True'''
    assert isinstance(c.plot, bool), \
        f'''(TypeError) 
        Plot type {c.plot} must be a boolean'''
    assert isinstance(c.save_figure, bool), \
        f'''(TypeError) 
        Save figure {c.save_figure} must be a boolean'''
    assert isinstance(c.figure_path, str), \
        f'''(TypeError)
        Figure path {c.figure_path} must be a string'''
    if c.save_figure:
        assert c.plot, \
            f'''(ValueError)
            Plot must be generated if save_figure=True'''
        assert len(c.figure_path) > 0, \
            f'''(ValueError)
            Figure path must be specified if save_figure=True'''
    
    return None

def check_amplitude_config(c):
    """Check attributes for class Amplitude"""
    
    assert isinstance(c.amp_factor, float), \
        f'''(TypeError)
        Amplitude factor {c.amp_factor} must be a float'''
    assert c.amp_factor > 0, \
        f'''(ValueError)
        Amplitude factor {c.amp_factor} must be greater than 0'''
    
    return None

def check_duration_config(c):
    """Check attributes for class Duration"""
    
    assert isinstance(c.moving_average_window, int), \
        f'''(TypeError) 
        Moving average {c.moving_average_window} must be an integer'''
    assert c.moving_average_window >= 0, \
        f'''(ValueError) 
        Moving average {c.moving_average_window} must be at least zero'''
    assert isinstance(c.start_fit_max, int), \
        f'''(TypeError) 
        Coda fitting start {c.start_fit_max} must be an integer'''
    assert c.start_fit_max > 0, \
        f'''(ValueError) 
        Coda fitting start {c.start_fit_max} must be positive'''
    assert isinstance(c.end_fit_noise, float), \
        f'''(TypeError) 
        Coda fitting end {c.end_fit_noise} must be a float'''
    assert c.end_fit_noise > 0, \
        f'''(ValueError) 
        Coda fitting end {c.end_fit_noise} must be positive'''
    assert isinstance(c.threshold_type, str), \
        f'''(TypeError) 
        Coda fitting threshold {c.threshold_type} must be a string'''
    assert c.threshold_type in ['absolute', 'noise'], \
        f'''(ValueError) 
        Coda fitting threshold {c.threshold_type} not recognized'''
    assert isinstance(c.duration_noise_threshold, float), \
        f'''(TypeError) 
        Duration threshold {c.duration_noise_threshold} must be a float'''
    assert isinstance(c.duration_absolute_threshold, float), \
        f'''(TypeError) 
        Duration threshold {c.duration_absolute_threshold} must be a float'''
    
    return None


def check_waveform(tr):
    """Perform various checks on the Trace"""
    
    check_stats(tr)
    check_datagaps(tr)
    
    return None
    
    
def check_stats(tr):
    """Check Trace stats"""
    
    if (hasattr(tr.stats, 'o') and 
        hasattr(tr.stats, 'p') and 
        hasattr(tr.stats, 's')):
        
        assert isinstance(tr.stats.o, obspy.UTCDateTime), \
            f'''(TypeError) 
            Origin time {tr.stats.o} must be an Obspy datetime object'''
        assert tr.stats.o - tr.stats.starttime >= 0, \
            f'''(ValueError) 
            Origin time {tr.stats.o} must be later than 
            Trace start {tr.stats.starttime}'''
        assert isinstance(tr.stats.p, float), \
            f'''(TypeError) 
            P- arrival {tr.stats.p} must be a float'''
        assert tr.stats.p > 0, \
            f'''(ValueError) 
            P- wave traveltime {tr.stats.p} must be positive'''
        assert isinstance(tr.stats.s, float), \
            f'''(TypeError) 
            S- arrival {tr.stats.s} must be a float'''
        assert tr.stats.s > tr.stats.p, \
            f'''(ValueError) 
            S- wave traveltime {tr.stats.s} must be greater than 
            P- wave traveltime {tr.stats.p}'''
        
    else:
        
        raise AttributeError('Arrivals missing from the Trace object')
        
        
    return None


def check_datagaps(tr, nsec=100):
    """Verify there is sufficient data"""
    
    datalength = len(tr.data) * tr.stats.delta
    assert datalength > nsec, \
        f'''(ValueError) 
        Insufficient data length ({datalength} seconds)'''
        
    return None
    
    
def check_window(tr, starttime, endtime):
    """Verify there's enough data for window definitions"""
    
    starttime_window = starttime - tr.stats.starttime
    endtime_window = tr.stats.endtime - endtime
    
    assert starttime_window > 0, \
        f'''(ValueError) 
        Window start ({starttime}) is earlier than 
        Trace start ({tr.stats.starttime})'''
    assert endtime_window > 0, \
        f'''(ValueError) 
        Window end ({endtime}) is later than 
        Trace end ({tr.stats.endtime})'''
        
    return None


def check_amplitude(amp):
    """Verify the amplitude value is real"""
    
    if ((isinstance(amp, float) == False and 
         isinstance(amp, int) == False) or 
         isinstance(amp, bool) == True or 
         np.isneginf(amp) or 
         np.isinf(amp) or 
         np.isnan(amp) or 
         amp < 0):
        
        raise ValueError(f'Invalid amplitude value: {amp}')
        
    return None


def check_plottype(ptype):
    """Verify the plottype"""
    
    assert isinstance(ptype, str), \
        f'''(TypeError) 
        Plot type {ptype} must be a string'''
    assert ptype in ['linear', 'log'], \
        f'''(ValueError) 
        Plot option {ptype} not recognized'''
    
    return None


def check_fitting_window_end(i_end, i_max, dt, sp):
    """Verify coda end is real and appropriate"""
    
    assert len(i_end) > 0, \
        '''(ValueError) 
        Insufficient length for coda fitting'''
    assert (i_end[0] + i_max) * dt > sp, \
        '''(ValueError) 
        Insufficient length for coda fitting'''

    return None


def check_coda(x0, y0):
    """Verify data does not have NaN values"""
    
    assert x0.dtype == float, \
        '''(TypeError) 
        Time must be a float array'''
    assert y0.dtype == float, \
        '''(TypeError) 
        Data must be a float array'''
    assert len(x0[np.isnan(x0)]) == 0, \
        '''(ValueError) 
        Time cannot have NaN values'''
    
    if len(y0[np.isnan(y0)]) > 0:
        x = x0[~np.isnan(y0)]
        y = y0[~np.isnan(y0)]
    else:
        x = np.copy(x0)
        y = np.copy(y0)
        
    return x, y


def check_duration_index(cross):
    """Verify the duration location"""
    
    assert len(cross) > 0, \
        '''(ValueError) 
        Coda fit line does not cross noise threshold'''
    
    return None


def check_cc(cc, x, y):
    """Verify the correlation coefficient exists"""
    
    assert cc.dtype == float, \
        '''(ValueError) 
        Correlation matrix must be float'''
    assert len(cc) == 4, \
        '''(ValueError) 
        Invalid correlation coefficient'''
    assert len(cc[0]) == 4, \
        '''(ValueError) 
        Invalid correlation coefficient'''
    assert ~np.isnan(cc[x, y]), \
        '''(ValueError) 
        Invalid correlation coefficient'''
    assert ~np.isinf(cc[x, y]), \
        '''(ValueError) 
        Invalid correlation coefficient'''
    assert ~np.isneginf(cc[x, y]), \
        '''(ValueError) 
        Invalid correlation coefficient'''
    assert cc[x, y] != 0, \
        f'''(ValueError) 
        Invalid correlation coefficient: {cc[x, y]}'''
    assert cc[x, y] >= -1, \
        f'''(ValueError) 
        Invalid correlation coefficient: {cc[x, y]}'''
    assert cc[x, y] <= 1, \
        f'''(ValueError) 
        Invalid correlation coefficient: {cc[x, y]}'''
    
    return None


import obspy
import unittest
import numpy as np
import madpy.amplitude as amp
import madpy.tests.testdata.config as cfg


class TestAmplitude(unittest.TestCase):
    
    
    def test_measure_amplitude(self):
        
        st = obspy.read('testdata/amplitude.mseed')
        st[0].stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
        st[0].stats.p = 10.
        st[0].stats.s = 20.
        
        df_amp = amp.measure_amplitude(st, cfg)
        self.assertEqual(len(df_amp), len(st))
        self.assertEqual(len(df_amp.columns), 7)
        
    
    def test_max_amplitude(self):
        
        st = obspy.read('testdata/amplitude.mseed')
        st[0].stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
        st[0].stats.p = 10.
        st[0].stats.s = 20.
        self.assertEqual(amp.max_amplitude(st[0], 0.1, cfg), 1.6)
        
        
    def test_trim_waveform_signal(self):
        
        st = obspy.read('testdata/amplitude.mseed')
        st[0].stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
        st[0].stats.p = 10.
        st[0].stats.s = 20.
        tr = amp.trim_waveform_signal(st[0], cfg)
        self.assertEqual(len(tr.data), 3101)
        
        
    def test_signal_window(self):
        
        st = obspy.read('testdata/amplitude.mseed')
        st[0].stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
        st[0].stats.p = 10.
        st[0].stats.s = 20.
        starttime, endtime = amp.signal_window(st[0], cfg)
        signal_begin = obspy.UTCDateTime('2020-10-10T13:05:19.00')
        signal_end = obspy.UTCDateTime('2020-10-10T13:05:50.00')
        self.assertEqual(starttime, signal_begin)
        self.assertEqual(endtime, signal_end)
        
        
    def test_inflection_points(self):
        
        data = np.array([0, -1, 0, 1, 0.5, -0.5, -1, -0.8, 0])
        data_ip = np.array([np.nan, -1, np.nan, 1, np.nan, np.nan, -1, np.nan, np.nan])
        ip = amp.inflection_points(data)
        self.assertEqual(len(ip[np.isnan(ip)]), 6)
        self.assertEqual(np.nansum(data_ip - ip), 0)
        
        
    def test_remove_nan(self):
        
        ip_nan = np.array([np.nan, -1, np.nan, 1, np.nan, np.nan, -1, np.nan, np.nan])
        data_ip = np.array([-1, 1, -1])
        ip = amp.remove_nan(data_ip)
        self.assertEqual(len(ip), 3)
        self.assertEqual(np.sum(data_ip - ip), 0)
        
        
    def test_p2p_indices(self):
        
        st = obspy.read('testdata/amplitude.mseed')
        starttime = obspy.UTCDateTime('2020-10-10T13:05:19.00')
        endtime = obspy.UTCDateTime('2020-10-10T13:05:50.00')
        st[0].trim(starttime=starttime, endtime=endtime)
        peaks_nan = amp.inflection_points(st[0].data.copy())
        peaks = amp.remove_nan(peaks_nan)
        idx = amp.p2p_indices(st[0].data.copy(), peaks, np.diff(peaks))
        self.assertEqual(idx[0], 1095)
        self.assertEqual(idx[1], 1104)
        
        
    def test_format_output(self):
        
        data = ['2020', '13:00:00', 1., 2., 3.]
        self.assertRaises(AssertionError, amp.format_output, data)
        

    
if __name__ == '__main__':
    unittest.main()


"""
amp.py

plot amplitude measurement
"""

import numpy as np
from matplotlib import markers
import matplotlib.pyplot as plt
from scipy.signal import hilbert
from matplotlib.path import Path
import madpy.plotting.utils as util
import madpy.plotting.params as params


PLOT_PHASE = 'O'

def amplitude_plot(tr_full, tr_signal, amp, indices, noise, cfg):
    """Generate amplitude plot"""
    
    # plot info
    time = util.to_time(tr_full, PLOT_PHASE)
    idx_phase = util.relative_phase_indices(tr_full, time, PLOT_PHASE)
    idx_p2p = relative_p2p_indices(tr_full, tr_signal, indices)
    xinfo = util.format_xaxis(time, -2, 5, 5, idx_phase[1], idx_p2p[1], '{:0.0f}')
    yinfo = format_amplitude_yaxis(time, tr_full, xinfo, 20, 2, '{:0.1E}')

    # plot parameters
    pp = params.amplitude_plot_parameters()
    xlabel = 'Time'
    ylabel = 'Ground motion'
    title = '{}: A = {:0.3f}, SNR = {:0.3f}'.format(
        tr_full.id, amp, amp / noise)    
    
    # plot
    fig, ax = plt.subplots(figsize=(8,4))
    ax.hlines(tr_full.data[idx_p2p[0]], xinfo['min'], xinfo['max'], 
              lw=pp['lw']['p2pl'], color=pp['c']['p2pl'], zorder=1)
    ax.hlines(tr_full.data[idx_p2p[1]], xinfo['min'], xinfo['max'], 
              lw=pp['lw']['p2pl'], color=pp['c']['p2pl'], zorder=2)
    ax.plot(time, tr_full.data, c=pp['c']['dat'], 
            lw=pp['lw']['dat'], zorder=3)
    ax.vlines(time[idx_phase[0]], yinfo['min'], yinfo['max'], 
              lw=pp['lw']['pha'], color=pp['c']['pha'], zorder=4)
    ax.vlines(time[idx_phase[1]], yinfo['min'], yinfo['max'], 
              lw=pp['lw']['pha'], color=pp['c']['pha'], zorder=5)
    ax.vlines(time[idx_phase[2]], yinfo['min'], yinfo['max'], 
              lw=pp['lw']['pha'], color=pp['c']['pha'], zorder=6)
    ax.plot(time[idx_p2p[0]], tr_full.data[idx_p2p[0]],
            marker=util.align_marker('^', valign='top'), c=pp['c']['p2pm'], 
            ms=18, alpha=0.5, zorder=7)
    ax.plot(time[idx_p2p[1]], tr_full.data[idx_p2p[1]],
            marker=util.align_marker('v', valign='bottom'), c=pp['c']['p2pm'], 
            ms=18, alpha=0.5, zorder=8)
    ax.set_xlabel(xlabel)
    ax.set_xlim(xinfo['min'], xinfo['max'])
    ax.set_xticks(xinfo['ticks'])
    ax.set_xticklabels(xinfo['ticklabels'])
    ax.set_ylabel(ylabel)
    ax.set_ylim(yinfo['min'], yinfo['max'])
    ax.set_yticks(yinfo['ticks'])
    ax.set_yticklabels(yinfo['ticklabels'])
    ax.set_title(title)
    plt.tight_layout()
    plt.close()
    
    if cfg.save_figure:
        fig.savefig(f'{cfg.figure_path}/amp-{tr_full.id}.png')
    
    
def relative_p2p_indices(tr_full, tr_signal, indices):
    """Get peak-to-peak indices relative to PLOT_PHASE"""
    
    i_signal = np.where(np.in1d(tr_full.data, tr_signal.data))[0]
    i_peak1 = i_signal[0] + indices[0]
    i_peak2 = i_signal[0] + indices[1]
    # TO-DO: check indices 
    indices_ordered = order_indices(tr_full, i_peak1, i_peak2) 
    
    return indices_ordered


def order_indices(tr, i_peak1, i_peak2):
    """Order indices such that the lowest one is first"""
    
    peak1 = tr.data[i_peak1]
    peak2 = tr.data[i_peak2]
    if peak1 > peak2:
        indices_ordered = np.array([i_peak2, i_peak1])
    else:
        indices_ordered = np.array([i_peak1, i_peak2])
        
    return indices_ordered


def format_amplitude_yaxis(time, tr, xinfo, yspace, nint, label_format):
    """Consolidate amplitude y-axis information"""
    
    i_xmin = np.where(np.abs(time - xinfo['min']) == \
                      np.nanmin(np.abs(time - xinfo['min'])))[0][0]
    i_xmax = np.where(np.abs(time - xinfo['max']) == \
                      np.nanmin(np.abs(time - xinfo['max'])))[0][0]
    ybig = np.max(np.abs(tr.data[i_xmin:i_xmax]))
    ymin = -ybig - ybig / yspace
    ymax_0 = ybig + ybig / yspace
    yint = ymax_0 / nint
    yticks = util.tick_info(ymin, ymax_0, yint, label_format)
    ymax = np.max(yticks[0])
    yinfo = {'min': ymin, 'max': ymax, 'int': yint,
             'ticks': yticks[0], 'ticklabels': yticks[1]}
    
    return yinfo


"""
dur.py

plot duration measurement
"""

import numpy as np
from matplotlib import markers
import matplotlib.pyplot as plt
from scipy.signal import hilbert
from matplotlib.path import Path
import madpy.duration as duration
import madpy.plotting.utils as util
import madpy.plotting.params as params


PLOT_PHASE = 'O'

def duration_plot(tr, avg_time, avg_data_lin, avg_data_log, 
                  fit_start, fit_end, dur, cc, 
                  coda, noise, ptype, cfg):
    """Generate duration plot"""
    
    # plot info
    time = util.to_time(tr, PLOT_PHASE)
    idx_phase = util.relative_phase_indices(tr, time, PLOT_PHASE)
    idx_coda = relative_coda_indices(time, avg_time, 
                                     fit_start, fit_end, idx_phase, dur)
    coda_line = coda_fit_line(time, coda, idx_coda, idx_phase[1])
    noise_threshold = duration.coda_line_end(cfg, noise)
    xinfo = util.format_xaxis(time, -1, 5, 5, 
                              idx_phase[0], idx_coda[2], '{:0.0f}')
    
    # plot parameters
    pp = params.duration_plot_parameters()
    xlabel = 'Time'
    title = r'{:s}: $\tau$ = {:0.3f} s, CC = –{:0.2f}'.format(tr.id, dur, np.abs(cc))
    if ptype == 'linear':
        full_data = tr.data
        avg_data = avg_data_lin
        yinfo = format_duration_yaxis(time, tr, xinfo, 20, 2, 
                                      ptype, '{:0.1E}')
        ylabel = 'Ground motion (linear)'
    elif ptype == 'log':
        full_data = duration.log_envelope(tr.data)
        avg_data = avg_data_log
        yinfo = format_duration_yaxis(time, tr, xinfo, 20, 4, 
                                      ptype, '{:0.0f}')
        ylabel = 'Ground motion (log)'
        
    # plot
    fig, ax = plt.subplots(figsize=(8,4))
    ax.plot(time, full_data, c=pp['c']['dat'], lw=pp['lw']['dat'], zorder=1)
    if cfg.moving_average_window > 0:
        ax.plot(avg_time + time[idx_phase[1]], avg_data, c=pp['c']['avg'], 
                lw=pp['lw']['avg'], zorder=2)
    ax.vlines(time[idx_phase[1]], yinfo['min'], yinfo['max'], 
              lw=pp['lw']['pha'], color=pp['c']['pha'], zorder=3)
    ax.vlines(time[idx_phase[2]], yinfo['min'], yinfo['max'], 
              lw=pp['lw']['pha'], color=pp['c']['pha'], zorder=4)
    ax.vlines(time[idx_coda[2]], yinfo['min'], yinfo['max'], 
              lw=pp['lw']['dur'], color=pp['c']['dur'], zorder=5)
    if ptype == 'log':
        ax.hlines(noise_threshold, xinfo['min'], xinfo['max'], 
                  lw=pp['lw']['nth'], color=pp['c']['nth'], zorder=6)
        ax.plot(time, coda_line[0], 
                lw=pp['lw']['fit'], color=pp['c']['fit'], zorder=7)
        if coda_line[1] is not None:
            ax.plot(time, coda_line[1], lw=pp['lw']['fit'], 
                    color=pp['c']['fit'], ls='--', zorder=8)
    ax.set_xlabel(xlabel)
    ax.set_xlim(xinfo['min'], xinfo['max'])
    ax.set_xticks(xinfo['ticks'])
    ax.set_xticklabels(xinfo['ticklabels'])
    ax.set_ylabel(ylabel)
    ax.set_ylim(yinfo['min'], yinfo['max'])
    ax.set_yticks(yinfo['ticks'])
    ax.set_yticklabels(yinfo['ticklabels'])
    ax.set_title(title)
    plt.tight_layout()
    plt.close()
    
    if cfg.save_figure:
        fig.savefig(f'{cfg.figure_path}/dur-{ptype}-{tr.id}.png')
    
               
def format_duration_yaxis(time, tr, xinfo, yspace, nint, ptype, label_format):
    """Consolidate duration y-axis information"""

    i_xmin = np.where(np.abs(time - xinfo['min']) == \
                      np.nanmin(np.abs(time - xinfo['min'])))[0][0]
    i_xmax = np.where(np.abs(time - xinfo['max']) == \
                      np.nanmin(np.abs(time - xinfo['max'])))[0][0]
    if ptype == 'linear':
        ybig = np.nanmax(np.abs(tr.data[i_xmin:i_xmax]))
        ymin = -ybig - ybig / yspace
        ymax_0 = ybig + ybig / yspace
        yint = ymax_0 / nint
        yticks = util.tick_info(ymin, ymax_0, yint, label_format)
        ymax = np.max(yticks[0])
        yinfo = {'min': ymin, 'max': ymax, 'int': yint,
                 'ticks': yticks[0], 'ticklabels': yticks[1]}
    elif ptype == 'log':
        data = duration.log_envelope(np.abs(hilbert(tr.data)))
        ymin = int(np.ceil(np.nanmin(data[i_xmin:i_xmax])))
        ymax = int(np.ceil(np.nanmax(data[i_xmin:i_xmax])))
        yint = (ymax - ymin) / nint
        yticks = util.tick_info(ymin, ymax, yint, label_format)
        yinfo = {'min': ymin, 'max': ymax, 'int': yint,
                 'ticks': yticks[0], 'ticklabels': yticks[1]}
    
    return yinfo    
    
    
def coda_fit_line(time, coda, idx_coda, i_p, extrapolation=None):
    """Get best fit line and whether or not it's extrapolated"""
    
    y_data = coda[1] * (time - time[i_p]) + coda[0]
    fit_line = np.empty(len(time),) * np.nan
    fit_line[idx_coda[0]:idx_coda[1]] = y_data[idx_coda[0]:idx_coda[1]]

    if idx_coda[2] > idx_coda[1]:
        extrapolation = np.empty(len(time),) * np.nan
        extrapolation[idx_coda[1]:idx_coda[2]] = \
        y_data[idx_coda[1]:idx_coda[2]]        
    
    return fit_line, extrapolation
        
        
def relative_coda_indices(time, avg_time, fit_start, fit_end, idx_phase, dur): 
    """Get indices of coda fit"""
    
    p_relative = time[idx_phase[1]]
    avg_time_rel = avg_time + p_relative
    start_rel = avg_time_rel[fit_start]
    end_rel = avg_time_rel[fit_end]
    dur_rel = dur + p_relative
    i_dur = np.where(np.abs(time - dur_rel) == \
                     np.nanmin(np.abs(time - dur_rel)))[0][0]
    i_start = np.where(np.abs(time - start_rel) == \
                       np.nanmin(np.abs(time - start_rel)))[0][0]
    i_end = np.where(np.abs(time - end_rel) == \
                     np.nanmin(np.abs(time - end_rel)))[0][0]
    # TO-DO: Check indices
    
    return i_start, i_end, i_dur


"""
generates the data
"""

import obspy
import numpy as np

def main():
    
    """
    generate test data
    """
    
    # General data
    npts = 30000
    dt = 0.01
    o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
    tr = generate_trace(npts, dt, o)
    
    # Amplitude data
    amp_data = amplitude_data(npts)
    tr_amp = tr.copy()
    tr_amp.data = amp_data
    tr_amp.write('amplitude.mseed', format='MSEED')
    
    # Duration data
    dur_data = duration_data(npts)
    tr_dur = tr.copy()
    tr_dur.data = dur_data
    tr_dur.write('duration.mseed', format='MSEED')
    
    
def duration_data(npts):
    
    """
    generate fake duration data
    """
    
    mean = 0
    stdev = 0.1
    m_coda, b_coda = -0.00025, -4.625
    m_p, b_p = 0.00267, -20.68
    p1, p2 = 4000, 5500
    c1, c2 = 5500, 21500
    base = -10
    log_data = np.zeros((npts + 1,)) + base
    log_data[p1:p2] = m_p * np.arange(p1, p2) + b_p
    log_data[c1:c2] = m_coda * np.arange(c1, c2) + b_coda
    log_data += np.random.normal(mean, stdev, npts + 1)
    lin_data = 10 ** log_data
    idx = np.random.choice(np.arange(len(lin_data)), int(0.5 * len(lin_data)))
    lin_data[idx] *= -1
    
    
    return lin_data
    
    
def amplitude_data(npts):
    
    """
    generate fake amplitude data
    """
    
    mean = 0
    stdev = 0.1
    p1, p2, pn = -1.5, 1.7, 10
    id1 = 5995
    data = np.random.normal(mean, stdev, npts + 1)
    data[id1:id1 + pn] = np.linspace(p1, p2, pn)
    
    return data    
    

def generate_trace(npts, dt, o):
    
    """
    This populates the trace
    """
    
    tr = obspy.Trace()
    tr.stats.network = 'IL'
    tr.stats.station = 'UIC'
    tr.stats.channel = 'SES'
    tr.stats.starttime = o - 30
    tr.stats.sampling_rate = 1/dt
    tr.stats.delta = dt
    tr.stats.npts = npts
    
    return tr
    
    

if __name__ == '__main__':
    main()




import obspy
import unittest
import numpy as np
import madpy.duration as dur
import madpy.tests.testdata.config as cfg


class TestAmplitude(unittest.TestCase):
    
    
    def test_measure_duration(self):
        
        st = obspy.read('testdata/duration.mseed')
        st[0].stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
        st[0].stats.p = 10.
        st[0].stats.s = 20.
        
        df_dur = dur.measure_duration(st, cfg)
        self.assertEqual(len(df_dur), len(st))
        self.assertEqual(len(df_dur.columns), 8) 
        
        
    def test_coda_duration(self):
        
        st = obspy.read('testdata/duration.mseed')
        st[0].stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
        st[0].stats.p = 10.
        st[0].stats.s = 20.

        duration, cc = dur.coda_duration(st[0], 10 ** -10, 'linear', cfg)
        self.assertAlmostEqual(duration, 175, 0)
        self.assertAlmostEqual(cc, -0.72, 2)
        
        
    def test_moving_average(self):
        
        st = obspy.read('testdata/duration.mseed')
        st[0].stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
        st[0].stats.p = 10.
        st[0].stats.s = 20.
        
        avg_time, avg_data = dur.moving_average(st[0].copy(), cfg.Duration)
        self.assertEqual(np.sum(st[0].data - avg_data), 0)
        cfg.Duration.moving_average_window = 2
        avg_time, avg_data = dur.moving_average(st[0].copy(), cfg.Duration)
        self.assertEqual(len(st[0].data) - len(avg_data), 
                         2 * st[0].stats.sampling_rate - 1)
        
        
    def test_time_relative_p(self):
        
        st = obspy.read('testdata/duration.mseed')
        st[0].stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
        st[0].stats.p = 10.
        st[0].stats.s = 20.
        
        time = np.arange(0, len(st[0].data)) * 0.01 - 40
        self.assertEqual(np.sum(dur.time_relative_p(st[0].copy()) - time), 0)
        
        
    def test_log_envelope(self):
        
        data = np.array([-10, 0, 1, 10, 100, 0, -10, 1, 100, 10], dtype=float)
        envelope = dur.log_envelope(data)
        self.assertEqual(len(envelope[np.isnan(envelope)]), 4)
        self.assertEqual(len(envelope[~np.isnan(envelope)]), 6)
        
        
    def test_coda_fitting_window(self):
        
        st = obspy.read('testdata/duration.mseed')
        st[0].stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
        st[0].stats.p = 10.
        st[0].stats.s = 20.
        
        time, data_0 = dur.moving_average(st[0].copy(), cfg.Duration)
        data = dur.log_envelope(data_0)
        i0, i1 = dur.coda_fitting_window(st[0].copy(), cfg.Duration, time, data, 10 ** -10)
        self.assertEqual(i0, 5631)
        self.assertEqual(i1, 19292)
        
        
    def test_search_window(self):
        
        st = obspy.read('testdata/duration.mseed')
        st[0].stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
        st[0].stats.p = 10.
        st[0].stats.s = 20.
        
        time,_ = dur.moving_average(st[0].copy(), cfg.Duration)
        begin = dur.search_window(st[0].copy(), time, cfg.Duration, 'begin')
        end = dur.search_window(st[0].copy(), time, cfg.Duration, 'end')
        self.assertEqual(begin, 4701)
        self.assertEqual(end, 6901)
    
        
    def test_search_window_seconds(self):
        
        st = obspy.read('testdata/duration.mseed')
        st[0].stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
        st[0].stats.p = 10.
        st[0].stats.s = 20.
        
        begin = dur.search_window_seconds(st[0].copy(), cfg.Duration, 'begin')
        end = dur.search_window_seconds(st[0].copy(), cfg.Duration, 'end')
        self.assertEqual(begin, 8)
        self.assertEqual(end, 30)
        
        
    def test_phase_relative_p(self):
        
        st = obspy.read('testdata/duration.mseed')
        st[0].stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
        st[0].stats.p = 10.
        st[0].stats.s = 20.
        
        self.assertEqual(dur.phase_relative_p(st[0], 'O'), -10)
        self.assertEqual(dur.phase_relative_p(st[0], 'P'), 0)
        self.assertEqual(dur.phase_relative_p(st[0], 'S'), 10)
        
        
    def test_fitting_window_start(self):
        
        self.assertEqual(dur.fitting_window_start(cfg.Duration, 20, 50), 20)
        cfg.Duration.start_fit_max = 3
        self.assertEqual(dur.fitting_window_start(cfg.Duration, 20, 50), 30)
        cfg.Duration.start_fit_max = 10
        self.assertEqual(dur.fitting_window_start(cfg.Duration, 20, 50), 23)
        
        
    def test_fitting_window_end(self):
        
        st = obspy.read('testdata/duration.mseed')
        st[0].stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
        st[0].stats.p = 10.
        st[0].stats.s = 20.
        
        _,data_0 = dur.moving_average(st[0].copy(), cfg.Duration)
        data = dur.log_envelope(data_0)
        end = dur.fitting_window_end(st[0], cfg.Duration, data, 5631, 10 ** -10)
        self.assertEqual(end, 19292)
        
        
    def test_coda_line_fit(self):
        
        m = -5
        b = 20
        x = np.arange(0, 1000, 0.1)
        y = m * x + b
        
        fit = dur.coda_line_fit(x, y)
        self.assertAlmostEqual(fit[1], m)
        self.assertAlmostEqual(fit[0], b)
        
        
    def test_get_duration_index(self):
        
        m = -5
        b = 20
        x = np.arange(0, 1000, 0.1)
        y = m * x + b
        
        i, cross = dur.get_duration_index(cfg.Duration, [b, m], 10 ** -10)
        self.assertAlmostEqual(i, 6.001)
        self.assertEqual(cross, 6001)
        
        
    def test_extend_line(self):
        
        x, y = dur.extend_line(-5, 20, 0, 500, 0.1)
        self.assertEqual(len(x), 500 / 0.1)
        self.assertEqual(len(y), 500 / 0.1)
         
        
    def test_coda_line_end(self):
        
        self.assertEqual(dur.coda_line_end(cfg.Duration, 10 ** -10), -10)
        cfg.Duration.threshold_type = 'noise'
        self.assertAlmostEqual(dur.coda_line_end(cfg.Duration, 10 ** -10), -20)
        cfg.Duration.threshold_type = 'absolute'
        
        
    def test_get_correlation_coefficient(self):
        
        line = np.array([-5, -1])
        time = np.arange(0, 1000, 0.01)
        data = line[1] * time + line[0]
        cc = dur.get_correlation_coefficient(line, time, data, 5, 90)
        self.assertAlmostEqual(cc, -1, 4)
        self.assertWarns(UserWarning, dur.get_correlation_coefficient, 
                         line, time, data, 5, 1110000)
               
        
    def test_calculate_cc(self):
        
        x = np.arange(0, 1000, 0.5)
        y = 5 * x - 1
        y_true = np.column_stack((x, y))
        self.assertRaises(AssertionError, dur.calculate_cc, y, y)
        self.assertEqual(dur.calculate_cc(y_true, y_true)[0, 3], 1)
        
        
    def format_output(self):
        
        data = ['2020', '13:00:00', 1., 2., 3.]
        self.assertRaises(AssertionError, dur.format_output, data)
        
    
if __name__ == '__main__':
    unittest.main()


from dataclasses import dataclass

@dataclass(init=False)
class Amplitude:
    noise_phase: str = 'P' 
    noise_window_begin: float = -30.
    noise_window_end: float =  -1.
    signal_phase: str = 'S'
    signal_window_begin: float = -1.
    signal_window_end: float = 30.
    amp_factor: float = 1.
    save_output: bool = True
    output_path: str = '.'
    plot: bool = True
    save_figure: bool = True
    figure_path: str = '.'

@dataclass(init=False)
class Duration:
    noise_phase: str = 'P'
    noise_window_begin: float = -30.
    noise_window_end: float = -1.
    signal_phase: str = 'S'
    signal_window_begin: float = -2.
    signal_window_end: float = 20.
    moving_average_window: int = 2
    start_fit_max: int = 1
    end_fit_noise: float = 2.
    threshold_type: str = 'absolute'
    duration_noise_threshold: float = 1.
    duration_absolute_threshold: float = -7.763462738511306
    save_output: bool = True
    output_path: str = '.'
    plot: bool = True
    save_figure: bool = True
    figure_path: str = '.'


"""
noise.py

measure the time series noise level
"""

# Installed 
import obspy
import types
import numpy as np
import pandas as pd
from typing import Tuple

# Local 
import madpy.checks as ch
import madpy.config as config


def rms_noise(
    tr: obspy.Trace, 
    measure: str, 
    cfg: types.ModuleType = config
) -> float:
    """Measure the noise level of the time series
    
    Args:
        tr: time series 
        measure: measurement type [amplitude, duration]
        cfg: configuration file
    
    Returns:
        noise: the time series noise level
        
    """
    
    tr_noise = trim_waveform_noise(tr.copy(), measure, cfg)
    noise = root_mean_square(tr_noise.data)
    
    return noise


def trim_waveform_noise(
    tr: obspy.Trace, 
    measure: str, 
    cfg: types.ModuleType = config
) -> obspy.Trace:  
    """Cut the time series to noise window
    
    Args:
        tr: time series
        measure: measurement type [amplitude, duration]
        cfg: configuration file
        
    Returns:
        tr: trimmed time series
        
    """
    
    starttime, endtime = noise_window(tr, measure, cfg)
    tr.trim(starttime=starttime, endtime=endtime)
        
    return tr


def noise_window(
    tr: obspy.Trace, 
    measure: str, 
    cfg: types.ModuleType = config
) -> Tuple[obspy.UTCDateTime, obspy.UTCDateTime]:    
    """Get the starttimes and endtimes of the noise window
    
    Args:
        tr: time series
        measure: measurement type [amplitude, duration]
        cfg: configuration file
        
    Returns:
        starttime: signal window beginning date
        endtime: signal window ending date
    
    Raises:
        AssertionError: Window begins before time series begins
        AssertionError: Window ends after time series ends
        
    """
    
    c = measurement_type(measure, cfg)  

    arrival = arrival_time_utc(tr, c.noise_phase)
    starttime = arrival + c.noise_window_begin
    endtime = arrival + c.noise_window_end
    ch.check_window(tr, starttime, endtime)
           
    return starttime, endtime
                                     
                    
def measurement_type(
    measure: str, 
    cfg: types.ModuleType = config
) -> config.Amplitude:    
    """Choose the appropriate class for measurement type
    Args:
        measure: measurement type [amplitude, duration]
        cfg: configuration file

    Returns:
        c: class instance            

    Raises:
        AssertionError: if measurement type is not amplitude or duration
            
    """
    
    assert measure in ['amplitude', 'duration']
    if measure == 'amplitude':
        c = cfg.Amplitude()
    if measure == 'duration':
        c = cfg.Duration()
    
    return c


def arrival_time_utc(tr: obspy.Trace, phase: str) -> obspy.UTCDateTime:   
    """Change arrival times to date format
    
    Args: 
        tr: time series
        phase: the seismic phase

    Returns:
        arrival: datetime format for phase
            
    """
   
    if phase == 'O':
        arrival = tr.stats.o
    elif phase == 'P':
        arrival = tr.stats.o + tr.stats.p
    elif phase == 'S':
        arrival = tr.stats.o + tr.stats.s
        
    return arrival


def root_mean_square(data: np.ndarray) -> float:
    """Calculate root mean square of time series
    
    Args:
        data: numpy array
    
    Returns:
        RMS value of the data

    """
    
    return np.sqrt(np.nanmean(data ** 2))


from setuptools import setup

setup(
    name='madpy-seis',
    version='0.1.0',    
    description='Measure Amplitude and Duration in Python',
    url='https://github.com/seismomomo/madpy',
    author='Monique Holt',
    author_email='mmholt@uic.edu',
    license='MIT',
    packages=['madpy', 'madpy.plotting', 'madpy.tests', 'madpy.tests.testdata'],
    install_requires=['jupyterlab==3.2.1',
                      'matplotlib==3.4.3',
                      'obspy==1.2.2',
                      'pandas==1.3.4',
                      'threadpoolctl==3.0.0'
                      ],

    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',  
        'Operating System :: POSIX :: Linux',        
        'Programming Language :: Python :: 3.9',
    ],
)



"""
utils.py

utility functions for plotting
"""

import numpy as np
import madpy.noise as n
from matplotlib import markers
import matplotlib.pyplot as plt
from scipy.signal import hilbert
from matplotlib.path import Path


def to_time(tr, phase):
    """create a time axis"""
    
    arrival = n.arrival_time_utc(tr, phase)
    shift = arrival - tr.stats.starttime
    samples = np.arange(0, len(tr)) * tr.stats.delta
    
    return samples - shift


def relative_phase_indices(tr, time, phase):
    """return indices for o, p, and s"""
    
    p_utc = n.arrival_time_utc(tr, 'P')
    s_utc = n.arrival_time_utc(tr, 'S')
    ref = n.arrival_time_utc(tr, phase)
    o_rel = tr.stats.o - ref
    p_rel = p_utc - ref
    s_rel = s_utc - ref
    i_o = np.where(np.abs(time - o_rel) == \
                   np.nanmin(np.abs(time - o_rel)))[0][0]
    i_p = np.where(np.abs(time - p_rel) == \
                   np.nanmin(np.abs(time - p_rel)))[0][0]
    i_s = np.where(np.abs(time - s_rel) == \
                   np.nanmin(np.abs(time - s_rel)))[0][0]
    # TO–DO: check indices
    
    return i_o, i_p, i_s


def format_xaxis(time, t1, t2, nint, ref1, ref2, label_format):
    """get xlimits and ticks relative to ref1 and ref2"""
    
    xmin = int(np.ceil(time[ref1] + t1))
    xmax_0 = int(np.floor(time[ref2] + t2))
    xint = int(np.round((xmax_0 - xmin) / 5))
    xticks = tick_info(xmin, xmax_0, xint, label_format)
    xmax = np.max(xticks[0])
    
    xinfo = {'min': xmin, 'max': xmax, 'int': xint,
             'ticks': xticks[0], 'ticklabels': xticks[1]}
    
    return xinfo
    
    
def tick_info(axmin, axmax, axint, label_format):
    """get tick info including labels"""
    
    axticks = np.arange(axmin, axmax + axint, axint)
    axticklabels = [label_format.format(tick) for tick in axticks]
    
    return axticks, axticklabels

    
def align_marker(marker, halign='center', valign='middle'):
    """Align matplotlib markers
    
    Code taken from 
    https://stackoverflow.com/questions/26686722/align-matplotlib-scatter-marker-left-and-or-right
    
    """

    if halign.isalpha():
        halign = {'right': -1.,
                  'middle': 0.,
                  'center': 0.,
                  'left': 1.,
                  }[halign]

    if valign.isalpha():
        valign = {'top': -1.,
                  'middle': 0.,
                  'center': 0.,
                  'bottom': 1.,
                  }[valign]

    bm = markers.MarkerStyle(marker)
    m_arr = bm.get_path().transformed(bm.get_transform()).vertices
    m_arr[:, 0] += halign / 2
    m_arr[:, 1] += valign / 2

    return Path(m_arr, bm.get_path().codes)


import obspy
import unittest
import numpy as np
import madpy.checks as ch
import madpy.tests.testdata.config as cfg


class TestChecks(unittest.TestCase):
            
    def test_check_config(self):
        
        class Measurements: pass
        self.assertRaises(AttributeError, ch.check_config, Measurements())
        self.assertIsNone(ch.check_config(cfg.Amplitude()))
        cfg.Amplitude.noise_phase = 'Pn'
        self.assertRaises(AssertionError, ch.check_config, cfg.Amplitude())
        cfg.Amplitude.noise_phase = 'P'
        cfg.Amplitude.amp_factor = -2.
        self.assertRaises(AssertionError, ch.check_config, cfg.Amplitude())
        cfg.Amplitude.plot = 'Yes'
        self.assertRaises(AssertionError, ch.check_config, cfg.Amplitude())
        cfg.Amplitude.plot = False
        cfg.Amplitude.signal_window_begin = 50.
        self.assertRaises(AssertionError, ch.check_config, cfg.Amplitude())
        cfg.Amplitude.signal_window_begin = -1
        cfg.Amplitude.save_figure = True
        self.assertRaises(AssertionError, ch.check_config, cfg.Amplitude())
        cfg.Amplitude.save_figure = False
        
        self.assertIsNone(ch.check_config(cfg.Duration()))
        cfg.Duration.signal_phase = 'Sg'
        self.assertRaises(AssertionError, ch.check_config, cfg.Duration())
        cfg.Duration.signal_phase = 'S'
        cfg.Duration.moving_average_window = -2
        self.assertRaises(AssertionError, ch.check_config, cfg.Duration())
        cfg.moving_average_window = 2
        cfg.threshold_type = 'pre-p noise'
        self.assertRaises(AssertionError, ch.check_config, cfg.Duration())
        cfg.threshold_type = 'noise'
        cfg.plot = True
        cfg.save_figure = True
        cfg.figure_path = ''
        self.assertRaises(AssertionError, ch.check_config, cfg.Duration())
        cfg.plot = False
        cfg.save_figure = False

        
    def test_check_waveform(self):
        
        st = obspy.read('testdata/*.mseed')
        for tr in st:
            tr.stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
            tr.stats.p = 10.
            self.assertRaises(AttributeError, ch.check_config, tr)
            tr.stats.s = 20.
            self.assertIsNone(ch.check_stats(tr))
            tr.stats.o = '2020-10-10T13:05:00.00'
            self.assertRaises(AssertionError, ch.check_stats, tr)
            tr.stats.o = obspy.UTCDateTime('2020-10-10T13:05:00.00')
        
        
    def test_check_datagaps(self):
        
        st = obspy.read('testdata/*.mseed')
        for tr in st:
            self.assertIsNone(ch.check_datagaps(tr))
            n = int(len(tr.data) * 0.25)
            tr.data = tr.data[0:n]
            self.assertRaises(AssertionError, ch.check_datagaps, tr) 
        
    
    def test_check_window(self):
        
        st = obspy.read('testdata/*.mseed')
        for tr in st:
            starttime = obspy.UTCDateTime('2020-10-10T13:05:00.00')
            endtime = obspy.UTCDateTime('2020-10-10T13:07:00.00')
            self.assertIsNone(ch.check_window(tr, starttime, endtime))
            starttime = obspy.UTCDateTime('2020-10-10T13:04:00.00')
            self.assertRaises(AssertionError, ch.check_window, tr, starttime, endtime)
            endtime = obspy.UTCDateTime('2020-10-10T13:08:00.00')
            self.assertRaises(AssertionError, ch.check_window, tr, starttime, endtime)
        
        
    def test_check_amplitude(self):
        
        self.assertIsNone(ch.check_amplitude(0.5))
        self.assertRaises(ValueError, ch.check_amplitude, np.nan)
        self.assertRaises(ValueError, ch.check_amplitude, np.inf)
        self.assertRaises(ValueError, ch.check_amplitude, -np.inf)
        self.assertRaises(ValueError, ch.check_amplitude, -0.5)
        self.assertRaises(ValueError, ch.check_amplitude, None)
        self.assertRaises(ValueError, ch.check_amplitude, True)
        self.assertRaises(ValueError, ch.check_amplitude, {'test': 'dict'})
        self.assertRaises(ValueError, ch.check_amplitude, ['list', 5])        
        
        
    def test_check_fitting_window_end(self):
        
        i_max0 = 20000
        i_end0 = np.arange(500, 5005)
        dt = 0.01
        sp = 10
        self.assertIsNone(ch.check_fitting_window_end(i_end0, i_max0, dt, sp))
        i_end1 = []
        self.assertRaises(AssertionError, ch.check_fitting_window_end, i_end1, i_max0, dt, sp)
        i_max1 = 2
        self.assertRaises(AssertionError, ch.check_fitting_window_end, i_end0, i_max1, dt, sp)
    
    
    def test_check_plottype(self):
        
        self.assertIsNone(ch.check_plottype('linear'))
        self.assertIsNone(ch.check_plottype('log'))
        self.assertRaises(AssertionError, ch.check_plottype, 2)
        self.assertRaises(AssertionError, ch.check_plottype, 'fourier')
        
        
    def test_check_duration_index(self):
        
        cross = np.arange(0, 10, dtype=float)
        self.assertIsNone(ch.check_duration_index(cross))
        self.assertRaises(AssertionError, ch.check_duration_index, [])      
    
    
    def test_check_cc(self):
        
        cc = np.array([
            [0.1, 0.8, 0.5, 0.9],
            [0.9, 0.1, 0.8, 0.5],
            [0.5, 0.9, 0.1, 0.8],
            [0.8, 0.5, 0.9, 0.1]
        ])
        self.assertIsNone(ch.check_cc(cc, 1, 2))
        self.assertRaises(AssertionError, ch.check_cc, cc.astype(int), 1, 2)
        self.assertRaises(AssertionError, ch.check_cc, cc[0:3, :], 1, 2)
        self.assertRaises(AssertionError, ch.check_cc, cc[:, 0:3], 1, 2)
        cc[1, 2] = np.nan
        self.assertRaises(AssertionError, ch.check_cc, cc, 1, 2)
        cc[1, 2] = np.inf
        self.assertRaises(AssertionError, ch.check_cc, cc, 1, 2)
        cc[1, 2] = -np.inf
        self.assertRaises(AssertionError, ch.check_cc, cc, 1, 2)
        cc[1, 2] = 0
        self.assertRaises(AssertionError, ch.check_cc, cc, 1, 2)
        cc[1, 2] = -10
        self.assertRaises(AssertionError, ch.check_cc, cc, 1, 2)
        cc[1, 2] = 25
        self.assertRaises(AssertionError, ch.check_cc, cc, 1, 2)
        
        
    def test_check_coda(self):
        
        x0 = np.arange(0, 100)
        y0 = np.arange(0, 100)
        x1 = np.arange(0, 100, dtype=float)
        y1 = np.arange(0, 100, dtype=float)
        self.assertRaises(AssertionError, ch.check_coda, x0, y1)
        self.assertRaises(AssertionError, ch.check_coda, x1, y0)
        self.assertRaises(AssertionError, ch.check_coda, x0, y0)
        x2, y2 = ch.check_coda(x1, y1)
        self.assertEqual(len(x2), 100)
        self.assertEqual(len(y2), 100)
        x2[5:10] = np.nan
        self.assertRaises(AssertionError, ch.check_coda, x2, y1)
        y2[60:72] = np.nan
        x3, y3 = ch.check_coda(x1, y2)
        self.assertEqual(len(x3), 88)
        self.assertEqual(len(y3), 88)
        
                
if __name__ == '__main__':
    unittest.main()


import matplotlib
params = {
    'text.latex.preamble': r'\usepackage{bm} \usepackage{amsmath}',
    'image.origin': 'lower',
    'image.interpolation': 'nearest',
    'image.cmap': 'gray',
    'savefig.dpi': 300,  
    'axes.labelsize': 14, 
    'axes.titlesize': 16,
    'axes.linewidth': 1,
    'font.size': 14, 
    'legend.fontsize': 12, 
    'xtick.labelsize': 12,
    'ytick.labelsize': 12,
    'xtick.major.width' : 1,
    'xtick.minor.width' : 0.5,
    'xtick.major.size' : 8,
    'xtick.minor.size' : 4,
    'ytick.major.width' : 1,
    'ytick.minor.width': 0.5,
    'ytick.major.size': 8,
    'ytick.minor.size': 4,
    'font.family': 'sans-serif',
    'errorbar.capsize': 6
    
}
matplotlib.rcParams.update(params)


def amplitude_plot_parameters():
    """Amplitude plot parameters"""
    
    colors = {
        'dat': 'dimgray',
        'pha': 'cornflowerblue',
        'p2pl': 'forestgreen',
        'p2pm': 'mediumvioletred'
    }
    
    linewidths = {
        'dat': 1,
        'pha': 2,
        'p2pl': 2
    }
    
    labels = {
        'dat': 'data',
        'o': 'o-arr',
        'p': 'p-arr',
        's': 's-arr',
        'p2pm': 'p2p'
    }

    
    return {'c': colors, 'lw': linewidths, 'lab': labels}



def duration_plot_parameters():
    """Duration plot parameters"""
    
    colors = {
        'dat': 'dimgray',
        'pha': 'cornflowerblue',
        'dur': 'darkblue',
        'avg': 'lavender',
        'nth': 'forestgreen',
        'fit': 'mediumvioletred'
    }
    
    linewidths = {
        'dat': 1,
        'pha': 2,
        'dur': 2,
        'avg': 2,
        'nth': 2,
        'fit': 2
    }
    
    labels = {
        'dat': 'data',
        'o': 'o-arr',
        'p': 'p-arr',
        's': 's-arr',
        'dur': 'dur',
        'avg': 'avg',
        'nth': 'thresh',
        'fit': 'linefit'
    }

    
    return {'c': colors, 'lw': linewidths, 'lab': labels}